package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.util.Arrays;
import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.CaseStatement;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.Statement;
import org.eclipse.jdt.internal.compiler.codegen.BranchLabel;
import org.eclipse.jdt.internal.compiler.codegen.CaseLabel;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.flow.FlowContext;
import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
import org.eclipse.jdt.internal.compiler.flow.SwitchFlowContext;
import org.eclipse.jdt.internal.compiler.flow.UnconditionalFlowInfo;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.SyntheticMethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;

public class SwitchStatement extends Statement {
  public Expression expression;
  
  public Statement[] statements;
  
  public BlockScope scope;
  
  public int explicitDeclarations;
  
  public BranchLabel breakLabel;
  
  public CaseStatement[] cases;
  
  public CaseStatement defaultCase;
  
  public int blockStart;
  
  public int caseCount;
  
  int[] constants;
  
  String[] stringConstants;
  
  public static final int CASE = 0;
  
  public static final int FALLTHROUGH = 1;
  
  public static final int ESCAPING = 2;
  
  private static final char[] SecretStringVariableName = " switchDispatchString".toCharArray();
  
  public SyntheticMethodBinding synthetic;
  
  int preSwitchInitStateIndex = -1;
  
  int mergedInitStateIndex = -1;
  
  CaseStatement[] duplicateCaseStatements = null;
  
  int duplicateCaseStatementsCounter = 0;
  
  private LocalVariableBinding dispatchStringCopy = null;
  
  public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
    try {
      FlowInfo flowInfo1;
      flowInfo = this.expression.analyseCode(currentScope, flowContext, flowInfo);
      if ((this.expression.implicitConversion & 0x400) != 0 || (
        this.expression.resolvedType != null && (
        this.expression.resolvedType.id == 11 || this.expression.resolvedType.isEnum())))
        this.expression.checkNPE(currentScope, flowContext, flowInfo, 1); 
      SwitchFlowContext switchContext = 
        new SwitchFlowContext(flowContext, (ASTNode)this, this.breakLabel = new BranchLabel(), true);
      UnconditionalFlowInfo unconditionalFlowInfo1 = FlowInfo.DEAD_END;
      this.preSwitchInitStateIndex = currentScope.methodScope().recordInitializationStates(flowInfo);
      int caseIndex = 0;
      if (this.statements != null) {
        int initialComplaintLevel = ((flowInfo.reachMode() & 0x3) != 0) ? 1 : 0;
        int complaintLevel = initialComplaintLevel;
        int fallThroughState = 0;
        for (int i = 0, max = this.statements.length; i < max; i++) {
          Statement statement = this.statements[i];
          if (caseIndex < this.caseCount && statement == this.cases[caseIndex]) {
            this.scope.enclosingCase = this.cases[caseIndex];
            caseIndex++;
            if (fallThroughState == 1 && (
              statement.bits & 0x20000000) == 0)
              this.scope.problemReporter().possibleFallThroughCase(this.scope.enclosingCase); 
            unconditionalFlowInfo1 = unconditionalFlowInfo1.mergedWith(flowInfo.unconditionalInits());
            complaintLevel = initialComplaintLevel;
            fallThroughState = 0;
          } else if (statement == this.defaultCase) {
            this.scope.enclosingCase = this.defaultCase;
            if (fallThroughState == 1 && (
              statement.bits & 0x20000000) == 0)
              this.scope.problemReporter().possibleFallThroughCase(this.scope.enclosingCase); 
            unconditionalFlowInfo1 = unconditionalFlowInfo1.mergedWith(flowInfo.unconditionalInits());
            complaintLevel = initialComplaintLevel;
            fallThroughState = 0;
          } else {
            fallThroughState = 1;
          } 
          if ((complaintLevel = statement.complainIfUnreachable((FlowInfo)unconditionalFlowInfo1, this.scope, complaintLevel, true)) < 2) {
            flowInfo1 = statement.analyseCode(this.scope, (FlowContext)switchContext, (FlowInfo)unconditionalFlowInfo1);
            if (flowInfo1 == FlowInfo.DEAD_END)
              fallThroughState = 2; 
            switchContext.expireNullCheckedFieldInfo();
          } 
        } 
      } 
      TypeBinding resolvedTypeBinding = this.expression.resolvedType;
      if (resolvedTypeBinding.isEnum()) {
        SourceTypeBinding sourceTypeBinding = (currentScope.classScope()).referenceContext.binding;
        this.synthetic = sourceTypeBinding.addSyntheticMethodForSwitchEnum(resolvedTypeBinding);
      } 
      if (this.defaultCase == null) {
        flowInfo.addPotentialInitializationsFrom((FlowInfo)flowInfo1.mergedWith(switchContext.initsOnBreak));
        this.mergedInitStateIndex = currentScope.methodScope().recordInitializationStates(flowInfo);
        return flowInfo;
      } 
      UnconditionalFlowInfo unconditionalFlowInfo2 = flowInfo1.mergedWith(switchContext.initsOnBreak);
      this.mergedInitStateIndex = 
        currentScope.methodScope().recordInitializationStates((FlowInfo)unconditionalFlowInfo2);
      return (FlowInfo)unconditionalFlowInfo2;
    } finally {
      if (this.scope != null)
        this.scope.enclosingCase = null; 
    } 
  }
  
  public void generateCodeForStringSwitch(BlockScope currentScope, CodeStream codeStream) {
    try {
      if ((this.bits & Integer.MIN_VALUE) == 0)
        return; 
      int pc = codeStream.position;
      boolean hasCases = (this.caseCount != 0);
      StringSwitchCase[] stringCases = new StringSwitchCase[this.caseCount];
      BranchLabel[] sourceCaseLabels = new BranchLabel[this.caseCount];
      CaseLabel[] hashCodeCaseLabels = new CaseLabel[this.caseCount];
      this.constants = new int[this.caseCount];
      for (int i = 0, max = this.caseCount; i < max; i++) {
        (this.cases[i]).targetLabel = sourceCaseLabels[i] = new BranchLabel(codeStream);
        (sourceCaseLabels[i]).tagBits |= 0x2;
        stringCases[i] = new StringSwitchCase(this, this.stringConstants[i].hashCode(), this.stringConstants[i], sourceCaseLabels[i]);
        hashCodeCaseLabels[i] = new CaseLabel(codeStream);
        (hashCodeCaseLabels[i]).tagBits |= 0x2;
      } 
      Arrays.sort((Object[])stringCases);
      int uniqHashCount = 0;
      int lastHashCode = 0;
      for (int j = 0, length = this.caseCount; j < length; j++) {
        int hashCode = (stringCases[j]).hashCode;
        if (j == 0 || hashCode != lastHashCode)
          lastHashCode = this.constants[uniqHashCount++] = hashCode; 
      } 
      if (uniqHashCount != this.caseCount) {
        System.arraycopy(this.constants, 0, this.constants = new int[uniqHashCount], 0, uniqHashCount);
        System.arraycopy(hashCodeCaseLabels, 0, hashCodeCaseLabels = new CaseLabel[uniqHashCount], 0, uniqHashCount);
      } 
      int[] sortedIndexes = new int[uniqHashCount];
      for (int k = 0; k < uniqHashCount; k++)
        sortedIndexes[k] = k; 
      CaseLabel defaultCaseLabel = new CaseLabel(codeStream);
      defaultCaseLabel.tagBits |= 0x2;
      this.breakLabel.initialize(codeStream);
      BranchLabel defaultBranchLabel = new BranchLabel(codeStream);
      if (hasCases)
        defaultBranchLabel.tagBits |= 0x2; 
      if (this.defaultCase != null)
        this.defaultCase.targetLabel = defaultBranchLabel; 
      this.expression.generateCode(currentScope, codeStream, true);
      codeStream.store(this.dispatchStringCopy, true);
      codeStream.addVariable(this.dispatchStringCopy);
      codeStream.invokeStringHashCode();
      if (hasCases) {
        codeStream.lookupswitch(defaultCaseLabel, this.constants, sortedIndexes, hashCodeCaseLabels);
        for (int m = 0, n = 0, i1 = this.caseCount; m < i1; m++) {
          int hashCode = (stringCases[m]).hashCode;
          if (m == 0 || hashCode != lastHashCode) {
            lastHashCode = hashCode;
            if (m != 0)
              codeStream.goto_(defaultBranchLabel); 
            hashCodeCaseLabels[n++].place();
          } 
          codeStream.load(this.dispatchStringCopy);
          codeStream.ldc((stringCases[m]).string);
          codeStream.invokeStringEquals();
          codeStream.ifne((stringCases[m]).label);
        } 
        codeStream.goto_(defaultBranchLabel);
      } else {
        codeStream.pop();
      } 
      int caseIndex = 0;
      if (this.statements != null)
        for (int m = 0, maxCases = this.statements.length; m < maxCases; m++) {
          Statement statement = this.statements[m];
          if (caseIndex < this.caseCount && statement == this.cases[caseIndex]) {
            this.scope.enclosingCase = this.cases[caseIndex];
            if (this.preSwitchInitStateIndex != -1)
              codeStream.removeNotDefinitelyAssignedVariables((Scope)currentScope, this.preSwitchInitStateIndex); 
            caseIndex++;
          } else if (statement == this.defaultCase) {
            defaultCaseLabel.place();
            this.scope.enclosingCase = this.defaultCase;
            if (this.preSwitchInitStateIndex != -1)
              codeStream.removeNotDefinitelyAssignedVariables((Scope)currentScope, this.preSwitchInitStateIndex); 
          } 
          statement.generateCode(this.scope, codeStream);
        }  
      if (this.mergedInitStateIndex != -1) {
        codeStream.removeNotDefinitelyAssignedVariables((Scope)currentScope, this.mergedInitStateIndex);
        codeStream.addDefinitelyAssignedVariables((Scope)currentScope, this.mergedInitStateIndex);
      } 
      codeStream.removeVariable(this.dispatchStringCopy);
      if (this.scope != currentScope)
        codeStream.exitUserScope(this.scope); 
      this.breakLabel.place();
      if (this.defaultCase == null) {
        codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd, true);
        defaultCaseLabel.place();
        defaultBranchLabel.place();
      } 
    } finally {
      if (this.scope != null)
        this.scope.enclosingCase = null; 
    } 
    if (this.scope != null)
      this.scope.enclosingCase = null; 
  }
  
  public void generateCode(BlockScope currentScope, CodeStream codeStream) {
    if (this.expression.resolvedType.id == 11) {
      generateCodeForStringSwitch(currentScope, codeStream);
      return;
    } 
    try {
      if ((this.bits & Integer.MIN_VALUE) == 0)
        return; 
      int pc = codeStream.position;
      this.breakLabel.initialize(codeStream);
      CaseLabel[] caseLabels = new CaseLabel[this.caseCount];
      for (int i = 0, max = this.caseCount; i < max; i++) {
        (this.cases[i]).targetLabel = (BranchLabel)(caseLabels[i] = new CaseLabel(codeStream));
        (caseLabels[i]).tagBits |= 0x2;
      } 
      CaseLabel defaultLabel = new CaseLabel(codeStream);
      boolean hasCases = (this.caseCount != 0);
      if (hasCases)
        defaultLabel.tagBits |= 0x2; 
      if (this.defaultCase != null)
        this.defaultCase.targetLabel = (BranchLabel)defaultLabel; 
      TypeBinding resolvedType = this.expression.resolvedType;
      boolean valueRequired = false;
      if (resolvedType.isEnum()) {
        codeStream.invoke((byte)-72, (MethodBinding)this.synthetic, null);
        this.expression.generateCode(currentScope, codeStream, true);
        codeStream.invokeEnumOrdinal(resolvedType.constantPoolName());
        codeStream.iaload();
        if (!hasCases)
          codeStream.pop(); 
      } else {
        valueRequired = !(this.expression.constant != Constant.NotAConstant && !hasCases);
        this.expression.generateCode(currentScope, codeStream, valueRequired);
      } 
      if (hasCases) {
        int[] sortedIndexes = new int[this.caseCount];
        for (int j = 0; j < this.caseCount; j++)
          sortedIndexes[j] = j; 
        int[] localKeysCopy;
        System.arraycopy(this.constants, 0, localKeysCopy = new int[this.caseCount], 0, this.caseCount);
        CodeStream.sort(localKeysCopy, 0, this.caseCount - 1, sortedIndexes);
        int k = localKeysCopy[this.caseCount - 1];
        int min = localKeysCopy[0];
        if ((long)(this.caseCount * 2.5D) > k - min) {
          if (k > 2147418112 && (currentScope.compilerOptions()).complianceLevel < 3145728L) {
            codeStream.lookupswitch(defaultLabel, this.constants, sortedIndexes, caseLabels);
          } else {
            codeStream.tableswitch(
                defaultLabel, 
                min, 
                k, 
                this.constants, 
                sortedIndexes, 
                caseLabels);
          } 
        } else {
          codeStream.lookupswitch(defaultLabel, this.constants, sortedIndexes, caseLabels);
        } 
        codeStream.recordPositionsFrom(codeStream.position, this.expression.sourceEnd);
      } else if (valueRequired) {
        codeStream.pop();
      } 
      int caseIndex = 0;
      if (this.statements != null)
        for (int j = 0, maxCases = this.statements.length; j < maxCases; j++) {
          Statement statement = this.statements[j];
          if (caseIndex < this.caseCount && statement == this.cases[caseIndex]) {
            this.scope.enclosingCase = this.cases[caseIndex];
            if (this.preSwitchInitStateIndex != -1)
              codeStream.removeNotDefinitelyAssignedVariables((Scope)currentScope, this.preSwitchInitStateIndex); 
            caseIndex++;
          } else if (statement == this.defaultCase) {
            this.scope.enclosingCase = this.defaultCase;
            if (this.preSwitchInitStateIndex != -1)
              codeStream.removeNotDefinitelyAssignedVariables((Scope)currentScope, this.preSwitchInitStateIndex); 
          } 
          statement.generateCode(this.scope, codeStream);
        }  
      if (this.mergedInitStateIndex != -1) {
        codeStream.removeNotDefinitelyAssignedVariables((Scope)currentScope, this.mergedInitStateIndex);
        codeStream.addDefinitelyAssignedVariables((Scope)currentScope, this.mergedInitStateIndex);
      } 
      if (this.scope != currentScope)
        codeStream.exitUserScope(this.scope); 
      this.breakLabel.place();
      if (this.defaultCase == null) {
        codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd, true);
        defaultLabel.place();
      } 
    } finally {
      if (this.scope != null)
        this.scope.enclosingCase = null; 
    } 
    if (this.scope != null)
      this.scope.enclosingCase = null; 
  }
  
  public StringBuffer printStatement(int indent, StringBuffer output) {
    printIndent(indent, output).append("switch (");
    this.expression.printExpression(0, output).append(") {");
    if (this.statements != null)
      for (int i = 0; i < this.statements.length; i++) {
        output.append('\n');
        if (this.statements[i] instanceof CaseStatement) {
          this.statements[i].printStatement(indent, output);
        } else {
          this.statements[i].printStatement(indent + 2, output);
        } 
      }  
    output.append("\n");
    return printIndent(indent, output).append('}');
  }
  
  public void resolve(BlockScope upperScope) {
    // Byte code:
    //   0: iconst_0
    //   1: istore_2
    //   2: iconst_0
    //   3: istore_3
    //   4: aload_0
    //   5: getfield expression : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   8: aload_1
    //   9: invokevirtual resolveType : (Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;)Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   12: astore #4
    //   14: aload_1
    //   15: invokevirtual compilerOptions : ()Lorg/eclipse/jdt/internal/compiler/impl/CompilerOptions;
    //   18: astore #5
    //   20: aload #4
    //   22: ifnull -> 200
    //   25: aload_0
    //   26: getfield expression : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   29: aload_1
    //   30: aload #4
    //   32: aload #4
    //   34: invokevirtual computeConversion : (Lorg/eclipse/jdt/internal/compiler/lookup/Scope;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   37: aload #4
    //   39: invokevirtual isValidBinding : ()Z
    //   42: ifne -> 51
    //   45: aconst_null
    //   46: astore #4
    //   48: goto -> 200
    //   51: aload #4
    //   53: invokevirtual isBaseType : ()Z
    //   56: ifeq -> 91
    //   59: aload_0
    //   60: getfield expression : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   63: aload #4
    //   65: getstatic org/eclipse/jdt/internal/compiler/lookup/TypeBinding.INT : Lorg/eclipse/jdt/internal/compiler/lookup/BaseTypeBinding;
    //   68: invokevirtual isConstantValueOfTypeAssignableToType : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)Z
    //   71: ifeq -> 77
    //   74: goto -> 200
    //   77: aload #4
    //   79: getstatic org/eclipse/jdt/internal/compiler/lookup/TypeBinding.INT : Lorg/eclipse/jdt/internal/compiler/lookup/BaseTypeBinding;
    //   82: invokevirtual isCompatibleWith : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)Z
    //   85: ifeq -> 184
    //   88: goto -> 200
    //   91: aload #4
    //   93: invokevirtual isEnum : ()Z
    //   96: ifeq -> 129
    //   99: iconst_1
    //   100: istore_2
    //   101: aload #5
    //   103: getfield complianceLevel : J
    //   106: ldc2_w 3211264
    //   109: lcmp
    //   110: ifge -> 200
    //   113: aload_1
    //   114: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   117: aload_0
    //   118: getfield expression : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   121: aload #4
    //   123: invokevirtual incorrectSwitchType : (Lorg/eclipse/jdt/internal/compiler/ast/Expression;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   126: goto -> 200
    //   129: aload_1
    //   130: aload #4
    //   132: getstatic org/eclipse/jdt/internal/compiler/lookup/TypeBinding.INT : Lorg/eclipse/jdt/internal/compiler/lookup/BaseTypeBinding;
    //   135: invokevirtual isBoxingCompatibleWith : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)Z
    //   138: ifeq -> 157
    //   141: aload_0
    //   142: getfield expression : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   145: aload_1
    //   146: getstatic org/eclipse/jdt/internal/compiler/lookup/TypeBinding.INT : Lorg/eclipse/jdt/internal/compiler/lookup/BaseTypeBinding;
    //   149: aload #4
    //   151: invokevirtual computeConversion : (Lorg/eclipse/jdt/internal/compiler/lookup/Scope;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   154: goto -> 200
    //   157: aload #5
    //   159: getfield complianceLevel : J
    //   162: ldc2_w 3342336
    //   165: lcmp
    //   166: iflt -> 184
    //   169: aload #4
    //   171: getfield id : I
    //   174: bipush #11
    //   176: if_icmpne -> 184
    //   179: iconst_1
    //   180: istore_3
    //   181: goto -> 200
    //   184: aload_1
    //   185: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   188: aload_0
    //   189: getfield expression : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   192: aload #4
    //   194: invokevirtual incorrectSwitchType : (Lorg/eclipse/jdt/internal/compiler/ast/Expression;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   197: aconst_null
    //   198: astore #4
    //   200: iload_3
    //   201: ifeq -> 250
    //   204: aload_0
    //   205: new org/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding
    //   208: dup
    //   209: getstatic org/eclipse/jdt/internal/compiler/ast/SwitchStatement.SecretStringVariableName : [C
    //   212: aload_1
    //   213: invokevirtual getJavaLangString : ()Lorg/eclipse/jdt/internal/compiler/lookup/ReferenceBinding;
    //   216: iconst_0
    //   217: iconst_0
    //   218: invokespecial <init> : ([CLorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;IZ)V
    //   221: putfield dispatchStringCopy : Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;
    //   224: aload_1
    //   225: aload_0
    //   226: getfield dispatchStringCopy : Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;
    //   229: invokevirtual addLocalVariable : (Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;)V
    //   232: aload_0
    //   233: getfield dispatchStringCopy : Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;
    //   236: getstatic org/eclipse/jdt/internal/compiler/impl/Constant.NotAConstant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   239: invokevirtual setConstant : (Lorg/eclipse/jdt/internal/compiler/impl/Constant;)V
    //   242: aload_0
    //   243: getfield dispatchStringCopy : Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;
    //   246: iconst_1
    //   247: putfield useFlag : I
    //   250: aload_0
    //   251: getfield statements : [Lorg/eclipse/jdt/internal/compiler/ast/Statement;
    //   254: ifnull -> 555
    //   257: aload_0
    //   258: new org/eclipse/jdt/internal/compiler/lookup/BlockScope
    //   261: dup
    //   262: aload_1
    //   263: invokespecial <init> : (Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;)V
    //   266: putfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   269: aload_0
    //   270: aload_0
    //   271: getfield statements : [Lorg/eclipse/jdt/internal/compiler/ast/Statement;
    //   274: arraylength
    //   275: dup
    //   276: istore #6
    //   278: anewarray org/eclipse/jdt/internal/compiler/ast/CaseStatement
    //   281: putfield cases : [Lorg/eclipse/jdt/internal/compiler/ast/CaseStatement;
    //   284: iload_3
    //   285: ifne -> 299
    //   288: aload_0
    //   289: iload #6
    //   291: newarray int
    //   293: putfield constants : [I
    //   296: goto -> 308
    //   299: aload_0
    //   300: iload #6
    //   302: anewarray java/lang/String
    //   305: putfield stringConstants : [Ljava/lang/String;
    //   308: iconst_0
    //   309: istore #7
    //   311: iconst_0
    //   312: istore #8
    //   314: goto -> 490
    //   317: aload_0
    //   318: getfield statements : [Lorg/eclipse/jdt/internal/compiler/ast/Statement;
    //   321: iload #8
    //   323: aaload
    //   324: astore #10
    //   326: aload #10
    //   328: aload_0
    //   329: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   332: aload #4
    //   334: aload_0
    //   335: invokevirtual resolveCase : (Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/ast/SwitchStatement;)Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   338: dup
    //   339: astore #9
    //   341: getstatic org/eclipse/jdt/internal/compiler/impl/Constant.NotAConstant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   344: if_acmpeq -> 487
    //   347: iload_3
    //   348: ifne -> 419
    //   351: aload #9
    //   353: invokevirtual intValue : ()I
    //   356: istore #11
    //   358: iconst_0
    //   359: istore #12
    //   361: goto -> 397
    //   364: aload_0
    //   365: getfield constants : [I
    //   368: iload #12
    //   370: iaload
    //   371: iload #11
    //   373: if_icmpne -> 394
    //   376: aload_0
    //   377: aload #10
    //   379: checkcast org/eclipse/jdt/internal/compiler/ast/CaseStatement
    //   382: aload_0
    //   383: getfield cases : [Lorg/eclipse/jdt/internal/compiler/ast/CaseStatement;
    //   386: iload #12
    //   388: aaload
    //   389: iload #6
    //   391: invokespecial reportDuplicateCase : (Lorg/eclipse/jdt/internal/compiler/ast/CaseStatement;Lorg/eclipse/jdt/internal/compiler/ast/CaseStatement;I)V
    //   394: iinc #12, 1
    //   397: iload #12
    //   399: iload #7
    //   401: if_icmplt -> 364
    //   404: aload_0
    //   405: getfield constants : [I
    //   408: iload #7
    //   410: iinc #7, 1
    //   413: iload #11
    //   415: iastore
    //   416: goto -> 487
    //   419: aload #9
    //   421: invokevirtual stringValue : ()Ljava/lang/String;
    //   424: astore #11
    //   426: iconst_0
    //   427: istore #12
    //   429: goto -> 468
    //   432: aload_0
    //   433: getfield stringConstants : [Ljava/lang/String;
    //   436: iload #12
    //   438: aaload
    //   439: aload #11
    //   441: invokevirtual equals : (Ljava/lang/Object;)Z
    //   444: ifeq -> 465
    //   447: aload_0
    //   448: aload #10
    //   450: checkcast org/eclipse/jdt/internal/compiler/ast/CaseStatement
    //   453: aload_0
    //   454: getfield cases : [Lorg/eclipse/jdt/internal/compiler/ast/CaseStatement;
    //   457: iload #12
    //   459: aaload
    //   460: iload #6
    //   462: invokespecial reportDuplicateCase : (Lorg/eclipse/jdt/internal/compiler/ast/CaseStatement;Lorg/eclipse/jdt/internal/compiler/ast/CaseStatement;I)V
    //   465: iinc #12, 1
    //   468: iload #12
    //   470: iload #7
    //   472: if_icmplt -> 432
    //   475: aload_0
    //   476: getfield stringConstants : [Ljava/lang/String;
    //   479: iload #7
    //   481: iinc #7, 1
    //   484: aload #11
    //   486: aastore
    //   487: iinc #8, 1
    //   490: iload #8
    //   492: iload #6
    //   494: if_icmplt -> 317
    //   497: iload #6
    //   499: iload #7
    //   501: if_icmpeq -> 580
    //   504: iload_3
    //   505: ifne -> 531
    //   508: aload_0
    //   509: getfield constants : [I
    //   512: iconst_0
    //   513: aload_0
    //   514: iload #7
    //   516: newarray int
    //   518: dup_x1
    //   519: putfield constants : [I
    //   522: iconst_0
    //   523: iload #7
    //   525: invokestatic arraycopy : (Ljava/lang/Object;ILjava/lang/Object;II)V
    //   528: goto -> 580
    //   531: aload_0
    //   532: getfield stringConstants : [Ljava/lang/String;
    //   535: iconst_0
    //   536: aload_0
    //   537: iload #7
    //   539: anewarray java/lang/String
    //   542: dup_x1
    //   543: putfield stringConstants : [Ljava/lang/String;
    //   546: iconst_0
    //   547: iload #7
    //   549: invokestatic arraycopy : (Ljava/lang/Object;ILjava/lang/Object;II)V
    //   552: goto -> 580
    //   555: aload_0
    //   556: getfield bits : I
    //   559: bipush #8
    //   561: iand
    //   562: ifeq -> 580
    //   565: aload_1
    //   566: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   569: aload_0
    //   570: getfield blockStart : I
    //   573: aload_0
    //   574: getfield sourceEnd : I
    //   577: invokevirtual undocumentedEmptyBlock : (II)V
    //   580: aload_0
    //   581: getfield defaultCase : Lorg/eclipse/jdt/internal/compiler/ast/CaseStatement;
    //   584: ifnonnull -> 627
    //   587: aload #5
    //   589: ldc_w 1073774592
    //   592: invokevirtual getSeverity : (I)I
    //   595: sipush #256
    //   598: if_icmpne -> 616
    //   601: iload_2
    //   602: ifeq -> 627
    //   605: aload_1
    //   606: invokevirtual methodScope : ()Lorg/eclipse/jdt/internal/compiler/lookup/MethodScope;
    //   609: iconst_1
    //   610: putfield hasMissingSwitchDefault : Z
    //   613: goto -> 627
    //   616: aload_1
    //   617: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   620: aload_0
    //   621: iload_2
    //   622: aload #4
    //   624: invokevirtual missingDefaultCase : (Lorg/eclipse/jdt/internal/compiler/ast/SwitchStatement;ZLorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   627: iload_2
    //   628: ifeq -> 860
    //   631: aload #5
    //   633: getfield complianceLevel : J
    //   636: ldc2_w 3211264
    //   639: lcmp
    //   640: iflt -> 860
    //   643: aload_0
    //   644: getfield defaultCase : Lorg/eclipse/jdt/internal/compiler/ast/CaseStatement;
    //   647: ifnull -> 658
    //   650: aload #5
    //   652: getfield reportMissingEnumCaseDespiteDefault : Z
    //   655: ifeq -> 860
    //   658: aload_0
    //   659: getfield constants : [I
    //   662: ifnonnull -> 669
    //   665: iconst_0
    //   666: goto -> 674
    //   669: aload_0
    //   670: getfield constants : [I
    //   673: arraylength
    //   674: istore #6
    //   676: iload #6
    //   678: aload_0
    //   679: getfield caseCount : I
    //   682: if_icmpne -> 860
    //   685: aload_0
    //   686: getfield caseCount : I
    //   689: aload #4
    //   691: checkcast org/eclipse/jdt/internal/compiler/lookup/ReferenceBinding
    //   694: invokevirtual enumConstantCount : ()I
    //   697: if_icmpeq -> 860
    //   700: aload #4
    //   702: invokevirtual erasure : ()Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   705: checkcast org/eclipse/jdt/internal/compiler/lookup/ReferenceBinding
    //   708: invokevirtual fields : ()[Lorg/eclipse/jdt/internal/compiler/lookup/FieldBinding;
    //   711: astore #7
    //   713: iconst_0
    //   714: istore #8
    //   716: aload #7
    //   718: arraylength
    //   719: istore #9
    //   721: goto -> 830
    //   724: aload #7
    //   726: iload #8
    //   728: aaload
    //   729: astore #10
    //   731: aload #10
    //   733: getfield modifiers : I
    //   736: sipush #16384
    //   739: iand
    //   740: ifne -> 746
    //   743: goto -> 827
    //   746: iconst_0
    //   747: istore #11
    //   749: goto -> 775
    //   752: aload #10
    //   754: getfield id : I
    //   757: iconst_1
    //   758: iadd
    //   759: aload_0
    //   760: getfield constants : [I
    //   763: iload #11
    //   765: iaload
    //   766: if_icmpne -> 772
    //   769: goto -> 827
    //   772: iinc #11, 1
    //   775: iload #11
    //   777: aload_0
    //   778: getfield caseCount : I
    //   781: if_icmplt -> 752
    //   784: aload_0
    //   785: getfield defaultCase : Lorg/eclipse/jdt/internal/compiler/ast/CaseStatement;
    //   788: ifnull -> 809
    //   791: aload_0
    //   792: getfield defaultCase : Lorg/eclipse/jdt/internal/compiler/ast/CaseStatement;
    //   795: getfield bits : I
    //   798: ldc_w 1073741824
    //   801: iand
    //   802: ifeq -> 809
    //   805: iconst_1
    //   806: goto -> 810
    //   809: iconst_0
    //   810: istore #11
    //   812: iload #11
    //   814: ifne -> 827
    //   817: aload_1
    //   818: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   821: aload_0
    //   822: aload #10
    //   824: invokevirtual missingEnumConstantCase : (Lorg/eclipse/jdt/internal/compiler/ast/SwitchStatement;Lorg/eclipse/jdt/internal/compiler/lookup/FieldBinding;)V
    //   827: iinc #8, 1
    //   830: iload #8
    //   832: iload #9
    //   834: if_icmplt -> 724
    //   837: goto -> 860
    //   840: astore #13
    //   842: aload_0
    //   843: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   846: ifnull -> 857
    //   849: aload_0
    //   850: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   853: aconst_null
    //   854: putfield enclosingCase : Lorg/eclipse/jdt/internal/compiler/ast/CaseStatement;
    //   857: aload #13
    //   859: athrow
    //   860: aload_0
    //   861: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   864: ifnull -> 875
    //   867: aload_0
    //   868: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   871: aconst_null
    //   872: putfield enclosingCase : Lorg/eclipse/jdt/internal/compiler/ast/CaseStatement;
    //   875: return
    // Line number table:
    //   Java source line number -> byte code offset
    //   #465	-> 0
    //   #466	-> 2
    //   #467	-> 4
    //   #468	-> 14
    //   #469	-> 20
    //   #470	-> 25
    //   #472	-> 37
    //   #473	-> 45
    //   #474	-> 48
    //   #475	-> 51
    //   #476	-> 59
    //   #477	-> 74
    //   #478	-> 77
    //   #479	-> 88
    //   #480	-> 91
    //   #481	-> 99
    //   #482	-> 101
    //   #483	-> 113
    //   #485	-> 126
    //   #486	-> 129
    //   #487	-> 141
    //   #488	-> 154
    //   #489	-> 157
    //   #490	-> 179
    //   #491	-> 181
    //   #493	-> 184
    //   #494	-> 197
    //   #497	-> 200
    //   #501	-> 204
    //   #502	-> 224
    //   #503	-> 232
    //   #504	-> 242
    //   #506	-> 250
    //   #507	-> 257
    //   #510	-> 269
    //   #511	-> 284
    //   #512	-> 288
    //   #513	-> 296
    //   #514	-> 299
    //   #516	-> 308
    //   #517	-> 311
    //   #519	-> 317
    //   #520	-> 326
    //   #521	-> 347
    //   #522	-> 351
    //   #524	-> 358
    //   #525	-> 364
    //   #526	-> 376
    //   #524	-> 394
    //   #529	-> 404
    //   #530	-> 416
    //   #531	-> 419
    //   #533	-> 426
    //   #534	-> 432
    //   #535	-> 447
    //   #533	-> 465
    //   #538	-> 475
    //   #517	-> 487
    //   #542	-> 497
    //   #543	-> 504
    //   #544	-> 508
    //   #545	-> 528
    //   #546	-> 531
    //   #549	-> 552
    //   #550	-> 555
    //   #551	-> 565
    //   #555	-> 580
    //   #556	-> 587
    //   #557	-> 601
    //   #558	-> 605
    //   #560	-> 613
    //   #561	-> 616
    //   #565	-> 627
    //   #566	-> 643
    //   #567	-> 658
    //   #568	-> 676
    //   #569	-> 685
    //   #570	-> 700
    //   #571	-> 713
    //   #572	-> 724
    //   #573	-> 731
    //   #575	-> 746
    //   #576	-> 752
    //   #577	-> 769
    //   #575	-> 772
    //   #580	-> 784
    //   #581	-> 812
    //   #582	-> 817
    //   #571	-> 827
    //   #589	-> 837
    //   #590	-> 842
    //   #591	-> 857
    //   #590	-> 860
    //   #592	-> 875
    // Local variable table:
    //   start	length	slot	name	descriptor
    //   0	876	0	this	Lorg/eclipse/jdt/internal/compiler/ast/SwitchStatement;
    //   0	876	1	upperScope	Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   2	835	2	isEnumSwitch	Z
    //   4	833	3	isStringSwitch	Z
    //   14	823	4	expressionType	Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   20	817	5	compilerOptions	Lorg/eclipse/jdt/internal/compiler/impl/CompilerOptions;
    //   278	274	6	length	I
    //   311	241	7	counter	I
    //   314	183	8	i	I
    //   341	146	9	constant	Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   326	161	10	statement	Lorg/eclipse/jdt/internal/compiler/ast/Statement;
    //   358	58	11	key	I
    //   361	43	12	j	I
    //   426	61	11	key	Ljava/lang/String;
    //   429	46	12	j	I
    //   676	161	6	constantCount	I
    //   713	124	7	enumFields	[Lorg/eclipse/jdt/internal/compiler/lookup/FieldBinding;
    //   716	121	8	i	I
    //   721	116	9	max	I
    //   731	96	10	enumConstant	Lorg/eclipse/jdt/internal/compiler/lookup/FieldBinding;
    //   749	35	11	j	I
    //   812	15	11	suppress	Z
    // Exception table:
    //   from	to	target	type
    //   0	840	840	finally
  }
  
  private void reportDuplicateCase(CaseStatement duplicate, CaseStatement original, int length) {
    if (this.duplicateCaseStatements == null) {
      this.scope.problemReporter().duplicateCase(original);
      this.scope.problemReporter().duplicateCase(duplicate);
      this.duplicateCaseStatements = new CaseStatement[length];
      this.duplicateCaseStatements[this.duplicateCaseStatementsCounter++] = original;
      this.duplicateCaseStatements[this.duplicateCaseStatementsCounter++] = duplicate;
    } else {
      boolean found = false;
      for (int k = 2; k < this.duplicateCaseStatementsCounter; k++) {
        if (this.duplicateCaseStatements[k] == duplicate) {
          found = true;
          break;
        } 
      } 
      if (!found) {
        this.scope.problemReporter().duplicateCase(duplicate);
        this.duplicateCaseStatements[this.duplicateCaseStatementsCounter++] = duplicate;
      } 
    } 
  }
  
  public void traverse(ASTVisitor visitor, BlockScope blockScope) {
    if (visitor.visit(this, blockScope)) {
      this.expression.traverse(visitor, blockScope);
      if (this.statements != null) {
        int statementsLength = this.statements.length;
        for (int i = 0; i < statementsLength; i++)
          this.statements[i].traverse(visitor, this.scope); 
      } 
    } 
    visitor.endVisit(this, blockScope);
  }
  
  public void branchChainTo(BranchLabel label) {
    if (this.breakLabel.forwardReferenceCount() > 0)
      label.becomeDelegateFor(this.breakLabel); 
  }
  
  public boolean doesNotCompleteNormally() {
    if (this.statements == null || this.statements.length == 0)
      return false; 
    for (int i = 0, length = this.statements.length; i < length; i++) {
      if (this.statements[i].breaksOut(null))
        return false; 
    } 
    return this.statements[this.statements.length - 1].doesNotCompleteNormally();
  }
  
  public boolean completesByContinue() {
    if (this.statements == null || this.statements.length == 0)
      return false; 
    for (int i = 0, length = this.statements.length; i < length; i++) {
      if (this.statements[i].completesByContinue())
        return true; 
    } 
    return false;
  }
}
